@@ -45,7 +45,7 @@
 #include <popt.h>
 
 #define URL_HANDLER_DIR      "/desktop/gnome/url-handlers/"
-#define DEFAULT_HANDLER_PATH "/desktop/gnome/url-handlers/default-show"
+#define DEFAULT_HANDLER_PATH "/desktop/gnome/url-handlers/unknown/command"
 
 /**
  * gnome_url_show
@@ -55,12 +55,12 @@
  *
  * Displays the given URL in an appropriate viewer. The appropriate viewer is
  * user definable. It is determined by extracting the protocol from the @url,
- * then seeing if the /desktop/gnome/url-handlers/<protocol>-show key exists in
+ * then seeing if the /desktop/gnome/url-handlers/<protocol>/command key exists in
  * the configuration database. It it does, this entry is used as the template
  * for the command. 
  *
  * If no protocol specific handler exists, the
- * /desktop/gnome/url-handlers/default-show key is used to determine the
+ * /desktop/gnome/url-handlers/unknown/command key is used to determine the
  * viewer.
  *
  * Once a viewer is determined, it is called with the @url as a parameter. If
@@ -78,9 +78,7 @@ gnome_url_show (const gchar *url, GError **error)
 	gchar *pos, *template;
 	int argc;
 	char **argv;
-	char **newargv;
 	gboolean ret;
-	gchar *tmp;
 	
 	g_return_val_if_fail (url != NULL, FALSE);
 
@@ -101,7 +99,7 @@ gnome_url_show (const gchar *url, GError **error)
 		protocol[pos - url] = '\0';
 		g_ascii_strdown (protocol, -1);
 
-		path = g_strconcat (URL_HANDLER_DIR, protocol, "-show", NULL);
+		path = g_strconcat (URL_HANDLER_DIR, protocol, "/command", NULL);
 		template = gconf_client_get_string (client, path, NULL);
 
 		if (template == NULL) {
@@ -133,8 +131,6 @@ gnome_url_show (const gchar *url, GError **error)
 
 	g_object_unref (G_OBJECT (client));
 
-	/* We use a popt function as it does exactly what we want to do and
-	   gnome already uses popt */
 	if (!g_shell_parse_argv (template,
 				 &argc,
 				 &argv,
@@ -143,37 +139,30 @@ gnome_url_show (const gchar *url, GError **error)
 		return FALSE;
 	}
 
-	newargv = g_new0 (char *, argc + 1);
-	for (i = 0; i < argc; i++) {
-		if (strcmp (argv[i], "%s") == 0)
-			newargv[i] = g_shell_quote (url);
-		else
-			newargv[i] = g_strdup (argv[i]);
-	}
-	newargv[i] = NULL;
-	
-	tmp = g_strjoinv (" ", newargv);
-	g_strfreev (newargv);
-	newargv = g_new0 (char *, 4);
-	newargv[0] = gnome_util_user_shell ();
-	newargv[1] = g_strdup ("-c");
-	newargv[2] = tmp;
-	newargv[3] = NULL;
+	g_free (template);
 
-	g_strfreev (argv);
+	for (i = 0; i < argc; i++) {
+		char *arg;
+
+		if (strcmp (argv[i], "%s") != 0)
+			continue;
+
+		arg = argv[i];
+		argv[i] = g_strdup (url);
+		g_free (arg);
+	}
 	
 	/* This can return some errors */
 	ret = g_spawn_async (NULL /* working directory */,
-			     newargv,
+			     argv,
 			     NULL /* envp */,
-			     0 /* flags */,
+			     G_SPAWN_SEARCH_PATH /* flags */,
 			     NULL /* child_setup */,
 			     NULL /* data */,
 			     NULL /* child_pid */,
 			     error);
 
-	g_strfreev (newargv);
-	g_free (template);
+	g_strfreev (argv);
 
 	return ret;
 }
